import random
import uuid
import datetime
from django.contrib.auth.models import AbstractUser, BaseUserManager
from django.db import models
from django.utils.timezone import now
from django.contrib.auth import get_user_model
from decimal import Decimal
from django.db.models.signals import post_save
from django.dispatch import receiver
# -----------------------------------------------
#  CUSTOM USER MANAGER (Handles User Creation)
# -----------------------------------------------
class UserManager(BaseUserManager):
    def create_user(self, email, password=None, **extra_fields):
        """Create and return a regular user with an email and password."""
        if not email:
            raise ValueError("The Email field must be set")
        email = self.normalize_email(email)
        extra_fields.setdefault("is_active", True)  # Ensure active by default
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password=None, **extra_fields):
        """Create and return a superuser with all permissions."""
        extra_fields.setdefault("is_staff", True)
        extra_fields.setdefault("is_superuser", True)
        extra_fields.setdefault("is_verified", True)  # Auto-verify superusers
        return self.create_user(email, password, **extra_fields)

# -----------------------------------------------
#  CUSTOM USER MODEL (Replaces Django’s Default)
# -----------------------------------------------
class User(AbstractUser):
    username = None  # Remove the default username field
    email = models.EmailField(unique=True)
    phone = models.CharField(max_length=15, unique=True)
    country = models.CharField(max_length=100)
    date_of_birth = models.DateField(null=True, blank=True)

    # National ID images for verification
    national_id_front = models.ImageField(upload_to="id_images/", blank=True, null=True)
    national_id_back = models.ImageField(upload_to="id_images/", blank=True, null=True)

    # Status fields
    is_active = models.BooleanField(default=True)  # Always active
    is_verified = models.BooleanField(default=False)  # Users need to verify email
    is_activated = models.BooleanField(default=False)  # Users must pay activation fee

    # OTP (One-Time Password) for authentication
    otp = models.CharField(max_length=6, blank=True, null=True)
    otp_expiry = models.DateTimeField(null=True, blank=True)

    # Password Reset models
    reset_token = models.CharField(max_length=100, blank=True, null=True)  
    reset_token_expiry = models.DateTimeField(null=True, blank=True)  

    objects = UserManager()  # Use custom manager

    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = ["phone"]

    def __str__(self):
        return f"{self.email} - Verified: {self.is_verified} - Activated: {self.is_activated}"


    def generate_otp(self):
        """Generate a 6-digit OTP with a 10-minute expiry."""
        self.otp = str(random.randint(100000, 999999))
        self.otp_expiry = now() + datetime.timedelta(minutes=10)
        self.save()

    def generate_reset_token(self):
        """Generate a secure password reset token valid for 1 hour."""
        self.reset_token = str(uuid.uuid4())
        self.reset_token_expiry = now() + datetime.timedelta(hours=1)
        self.save()

# -----------------------------------------------
# ACCOUNT MODEL (Manages User Balances)
# -----------------------------------------------
class Account(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE, related_name="account")
    balance = models.DecimalField(max_digits=10, decimal_places=2, default=0.00)

    # Payment Methods
    credit_card_number = models.CharField(max_length=16, blank=True, null=True)  # Store securely
    credit_card_expiry = models.CharField(max_length=5, blank=True, null=True)  # Format: MM/YY
    credit_card_cvv = models.CharField(max_length=3, blank=True, null=True)

    bank_name = models.CharField(max_length=100, blank=True, null=True)
    account_number = models.CharField(max_length=20, blank=True, null=True)
    routing_number = models.CharField(max_length=9, blank=True, null=True)

    def deposit(self, amount):
        """Add funds to the user’s balance."""
        amount = Decimal(amount) 
        self.balance += amount
        self.save()

    def withdraw(self, amount):
        """Withdraw funds if sufficient balance exists and a payment method is set."""
        amount = Decimal(amount) 
        if self.balance >= amount and self.has_payment_method():
            self.balance -= amount
            self.save()
            return True
        return False

    def has_payment_method(self):
        """Ensure the user has a valid payment method before allowing withdrawals."""
        return bool(self.credit_card_number or self.account_number)

    def external_deposit(self, amount, source):
        """Add funds to the account from an external source like Global Tasks."""
        amount = Decimal(amount)
        self.balance += amount
        self.save()

        # Log the external deposit in the transaction table
        Transaction.objects.create(
            sender=None,  # No sender for external payments
            recipient=self.user,
            transaction_type='deposit',
            amount=amount,
            timestamp=now()
        )

        return self.balance


    def __str__(self):
        return f"{self.user.email} - Balance: ${self.balance}"

# -----------------------------------------------
#  TRANSACTION MODEL (Tracks User Transactios) that occure
# -----------------------------------------------
class Transaction(models.Model):
    TRANSACTION_TYPES = [
        ("deposit", "Deposit"),
        ("withdraw", "Withdraw"),
        ("transfer", "Transfer"),
    ]

    sender = models.ForeignKey(User, on_delete=models.SET_NULL, null=True, related_name="sent_transactions", blank=True)
    recipient = models.ForeignKey(User, on_delete=models.SET_NULL, null=True, related_name="received_transactions", blank=True)
    transaction_type = models.CharField(max_length=10, choices=TRANSACTION_TYPES)
    amount = models.DecimalField(max_digits=10, decimal_places=2)
    timestamp = models.DateTimeField(default=now)
    source = models.CharField(max_length=255, blank=True, null=True) 

    def __str__(self):
        return f"{self.transaction_type} - ${self.amount} on {self.timestamp.strftime('%Y-%m-%d %H:%M:%S')}"

class PaymentActivation(models.Model):
    user = models.OneToOneField(User, on_delete=models.CASCADE, related_name="activation_payment")
    amount = models.DecimalField(max_digits=5, decimal_places=2, default=2.40)
    is_paid = models.BooleanField(default=False)
    payment_date = models.DateTimeField(null=True, blank=True)
    
 

    # @receiver(post_save, sender=User)
    # def create_payment_activation(sender, instance, created, **kwargs):
    #     """Automatically creates a PaymentActivation record when a new user is created."""
    #     if created:
    #         PaymentActivation.objects.create(user=instance)


    def mark_paid(self):
        """Marks the activation fee as paid and activates the user"""
        self.is_paid = True
        self.payment_date = now()
        self.save()
        self.user.is_activated = True  # Account is activated after payment
        self.user.save()

    def __str__(self):
        return f"Activation Payment - {self.user.email} - Paid: {self.is_paid}"
# -----------------------------------------------
#  NOTIFICATION MODEL (Stores User Notifications)
# -----------------------------------------------
class Notification(models.Model):
    NOTIFICATION_TYPES = [
        ("info", "Information"),
        ("warning", "Warning"),
        ("error", "Error"),
        ("success", "Success"),
        ("custom", "Custom"),
    ]
    
    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE, related_name="notifications")
    notification_type = models.CharField(max_length=10, choices=NOTIFICATION_TYPES)
    message = models.TextField()  # Notification message
    is_read = models.BooleanField(default=False)  # Flag for read/unread status
    timestamp = models.DateTimeField(default=now)  # When the notification was created
    is_active = models.BooleanField(default=True)  # Flag to mark active notifications
    
    def mark_as_read(self):
        """Marks the notification as read."""
        self.is_read = True
        self.save()

    def mark_as_unread(self):
        """Marks the notification as unread."""
        self.is_read = False
        self.save()

    def __str__(self):
        return f"Notification for {self.user.email} - {self.notification_type} - {self.is_read}"

